@@ -1,7 +1,9 @@
 """ Converter routine to create a sweep definition from a given DataFrame
 """
 # DMT_core
-# Copyright (C) 2019  Markus Müller and Mario Krattenmacher and the DMT contributors <https://gitlab.hrz.tu-chemnitz.de/CEDIC_Bipolar/DMT/>
+# Copyright (C) from 2022  SemiMod
+# Copyright (C) until 2021  Markus Müller, Mario Krattenmacher and Pascal Kuthe
+# <https://gitlab.com/dmt-development/dmt-core>
 #
 # This file is part of DMT_core.
 #
@@ -17,30 +19,50 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>
+from __future__ import annotations
+from typing import Type, Optional
+import warnings
 import numpy as np
 
-from DMT.core import Sweep, specifiers, sub_specifiers
+from DMT.core import Sweep, specifiers, sub_specifiers, DataFrame, SweepDef
 
 
-def df_to_sweep(df_to_convert, temperature=300, name=None, from_forced=True, SweepDefClass=None):
+def df_to_sweep(
+    df_to_convert: DataFrame,
+    temperature: float = 300.0,
+    name: Optional[str] = None,
+    from_forced: bool = True,
+    SweepDefClass: Type = SweepDef,
+    decimals_potentials: int = 3,
+):
     """Create a sweepdefinition which simulates the same forced values as the given measurement data frame.
 
     For the moment only voltage sweeps...
 
     Parameters
-    ------------
-    df_to_convert : :class:`DMT.core.data_frame.DataFrame`
+    ----------
+    df_to_convert : :class:`~DMT.core.data_frame.DataFrame`
+        DataFrame with voltages which define the sweep.
     temperature : float, optional
-        Temperature of the measurement in Kelvin.
+        Temperature of the measurement in Kelvin, defaults to 300.0
     name : str, optional
         Name of the sweep to create. If None, automatic generation of a name is tried.
     from_forced : {True, False}, optional
         If True, the forced voltages are used to create the sweepdef.
+    SweepDefClass : :class:`~DMT.core.sweep.SweepDef`
+        SweepDef class for the sweep to create from
+    decimals_potentials : int
+        Round the potentials to this x decimals, defaults to 3
 
     Returns
-    ---------
-    sweep : :class:`DMT.core.sweep.Sweep`
+    -------
+    :class:`~DMT.core.sweep.Sweep`
     """
+    warnings.warn(
+        "df_to_sweep is deprecated and will be removed in the next major release.\nUse Sweep.get_sweep in future.\n",
+        category=DeprecationWarning,
+    )
+
     vars_other = {
         specifiers.TEMPERATURE: temperature
     }  # temperature is usually in the dut.data key and not in the df. Additionally it is always constant ?!?
@@ -72,7 +94,7 @@ def df_to_sweep(df_to_convert, temperature=300, name=None, from_forced=True, Swe
     for potential in sorted(
         potentials_forced, key=lambda x: x.nodes[0]
     ):  # sort to make sure everytime the same is found.
-        if len(np.unique(np.round(df_to_convert[potential].to_numpy(), 6))) == 1:
+        if len(np.unique(np.round(df_to_convert[potential].to_numpy(), decimals_potentials))) == 1:
             potential_common = potential  # use this one! Even if more than one would exist.
             break
 
@@ -112,14 +134,14 @@ def df_to_sweep(df_to_convert, temperature=300, name=None, from_forced=True, Swe
                 voltage = potential + potential_other.nodes[0]
 
             df_to_convert.ensure_specifier_column(voltage)
-            values = np.unique(np.round(df_to_convert[voltage].to_numpy(), 6))
+            values = np.unique(np.round(df_to_convert[voltage].to_numpy(), decimals_potentials))
             count_ops[voltage] = len(values)
 
     for i_voltage, (voltage, _count) in enumerate(
         sorted(count_ops.items(), key=lambda kv: kv[1])[:-1]
     ):  # last of the sorted list is the one we don't need
         df_to_convert.ensure_specifier_column(voltage)
-        values = np.unique(np.round(df_to_convert[voltage].to_numpy(), 6))
+        values = np.unique(np.round(df_to_convert[voltage].to_numpy(), decimals_potentials))
 
         if voltage.nodes[1] == potential_common.nodes[0]:  # convert to potential if possible
             if from_forced:
@@ -143,6 +165,8 @@ def df_to_sweep(df_to_convert, temperature=300, name=None, from_forced=True, Swe
             ]:
                 other_nodes += "".join(voltage_other.nodes)
 
+            potential = specifiers.VOLTAGE + "C"  # per default a BC sweep
+            potential_master = specifiers.VOLTAGE + "B"
             for node_master in voltage.nodes:
                 if node_master in other_nodes:
                     if from_forced:
@@ -157,7 +181,7 @@ def df_to_sweep(df_to_convert, temperature=300, name=None, from_forced=True, Swe
             voltage_offset = potential + potential_master.nodes[0]
 
             if voltage != voltage_offset:  # order of nodes reversed
-                values = np.round(-values, 6)  # invert voltage
+                values = np.round(-values, decimals_potentials)  # invert voltage
 
             def_sweep.append(
                 {
@@ -194,7 +218,9 @@ def df_to_sweep(df_to_convert, temperature=300, name=None, from_forced=True, Swe
 
     ### grab definition for the frequency, if possible
     if specifiers.FREQUENCY in df_to_convert.columns:
-        freq = np.unique(np.round(df_to_convert[specifiers.FREQUENCY].to_numpy(), 6))
+        freq = np.unique(
+            np.round(df_to_convert[specifiers.FREQUENCY].to_numpy(), decimals_potentials)
+        )
         def_sweep.append(
             {
                 "var_name": specifiers.FREQUENCY,
