@@ -7,11 +7,13 @@ Features:
 * Supports to run simulations on multiple cores in parallel
 * Supports to run simulations on a remote server (including file up- and download)
 
-Author: Mario Krattenmacher | Mario.Krattenmacher@tu-dresden.de
-Author: Markus Müller | markus.mueller3@tu-dresden.de
+Author: Mario Krattenmacher | mario.krattenmacher@semimod.de
+Author: Markus Müller | markus.mueller@semimod.de
 """
 # DMT_core
-# Copyright (C) 2019  Markus Müller and Mario Krattenmacher and the DMT contributors <https://gitlab.hrz.tu-chemnitz.de/CEDIC_Bipolar/DMT/>
+# Copyright (C) from 2022  SemiMod
+# Copyright (C) until 2021  Markus Müller, Mario Krattenmacher and Pascal Kuthe
+# <https://gitlab.com/dmt-development/dmt-core>
 #
 # This file is part of DMT_core.
 #
@@ -29,7 +31,6 @@ Author: Markus Müller | markus.mueller3@tu-dresden.de
 # along with this program.  If not, see <http://www.gnu.org/licenses/>
 import copy
 import logging
-import os
 import time
 import subprocess
 import itertools
@@ -45,8 +46,12 @@ from DMT.exceptions import SimulationUnsuccessful, SimulationFail
 # import them always -> can become very annoying otherways (if default is False but one dut is remote)
 from tempfile import NamedTemporaryFile
 from zipfile import ZipFile
-import paramiko
-from scp import SCPClient, SCPException
+
+try:
+    import paramiko
+    from scp import SCPClient, SCPException
+except ImportError:
+    pass
 
 
 def upload_progress(filename, size, sent):
@@ -58,19 +63,19 @@ class SimCon(object, metaclass=Singleton):
     """Simulation controller class. SINGLETON design pattern.
 
     Parameters
-    -----------
+    ----------
     n_core  :  int
         Number of cores that shall be used for simulations.
     t_max   :  float
         Timeout for simulations. If a simulation runs longer than t_max in seconds, it is killed.
 
     Attributes
-    -----------
+    ----------
     n_core  :  int
         Number of cores that shall be used for simulations.
     t_max   :  float
         Timeout for simulations. If a simulation runs longer than t_max in seconds, it is killed.
-    sim_list :  [{'dut': dut, 'sweep': sweep}]
+    sim_list :  [{'dut': :class:`~DMT.core.dut_view.DutView`, 'sweep': :class:`~DMT.core.sweep.Sweep`}]
         A list of dicts containing the queued simulations. Each dict holds a 'dut' key value pair and a 'sweep' key value pair.
 
     ssh_client
@@ -102,7 +107,7 @@ class SimCon(object, metaclass=Singleton):
         This methods adds each dut with a copy of each sweep to the simulation list.
 
         Parameters
-        -----------
+        ----------
         dut : :class:`~DMT.core.dut_view.DutView` or [:class:`~DMT.core.dut_view.DutView`]
             Objected of a subclass of DutView. This object describes the device to be simulated and specifies the backend.
         sweep : :class:`~DMT.core.sweep.Sweep` or [:class:`~DMT.core.sweep.Sweep`]
@@ -243,6 +248,7 @@ class SimCon(object, metaclass=Singleton):
             DATA_CONFIG["server"]["adress"],
             username=DATA_CONFIG["server"]["ssh_user"],
             key_filename=str(Path(DATA_CONFIG["server"]["ssh_key"]).expanduser()),
+            disabled_algorithms=dict(pubkeys=["rsa-sha2-512", "rsa-sha2-256"]),
         )
 
         self.scp_client = SCPClient(
@@ -261,8 +267,8 @@ class SimCon(object, metaclass=Singleton):
 
         # make sure that the target folder exists
         for folder in reversed(DATA_CONFIG["server"]["simulation_path"].parents):
-            self.ssh_client.exec_command("mkdir " + str(folder))
-        self.ssh_client.exec_command("mkdir " + str(DATA_CONFIG["server"]["simulation_path"]))
+            self.ssh_client.exec_command("mkdir -p " + str(folder))
+        self.ssh_client.exec_command("mkdir -p " + str(DATA_CONFIG["server"]["simulation_path"]))
 
     def close_ssh_client(self):
         """Closes the ssh connection again."""
@@ -275,7 +281,7 @@ class SimCon(object, metaclass=Singleton):
         """Copies the simulation data to the server. Before doing this, old simulation data is deleted
 
         Parameters
-        -----------
+        ----------
         sims_to_zip : list[dict]
             A list of dictionaries with 2 keys:
             dut : DutView
@@ -292,16 +298,20 @@ class SimCon(object, metaclass=Singleton):
                 else:
                     add_to_zip(child, rel_to)
 
+        assert self.ssh_client is not None
+        assert self.scp_client is not None
+
         sim_path_on_server = DATA_CONFIG["server"]["simulation_path"]
-        commands = ""
+        commands = []
         # delete possible old directories:
         for sim_to_zip in sims_to_zip:
             sim_folder = sim_to_zip["dut"].get_sim_folder(sim_to_zip["sweep"])
             dut_folder = sim_folder.parts[-2]
             sweep_folder = sim_folder.parts[-1]
-            commands += (
-                "rm -rf " + str(sim_path_on_server / dut_folder / sweep_folder) + "\n"
-            )  # remove to be sure
+            commands.append("rm -rf " + str(sim_path_on_server / dut_folder / sweep_folder))
+
+        for command in commands:
+            self.ssh_client.exec_command(command)
 
         with NamedTemporaryFile() as path_zip:  # dut.get_sim_folder(sweep).relative_to(dut.sim_dir)
             with ZipFile(path_zip, "w") as zip_ref:
@@ -310,25 +320,28 @@ class SimCon(object, metaclass=Singleton):
                         sim_to_zip["dut"].get_sim_folder(sim_to_zip["sweep"]),
                         sim_to_zip["dut"].sim_dir,
                     )
+                    # add "central" VA-Files -.-
+                    if not sim_to_zip["dut"]._copy_va_files:
+                        va_files_dir = sim_to_zip["dut"].sim_dir / "VA_codes"
+                        for vafile in sim_to_zip["dut"]._list_va_file_contents:
+                            dir_code = va_files_dir / vafile.get_tree_hash()
+                            add_to_zip(
+                                dir_code,
+                                sim_to_zip["dut"].sim_dir,
+                            )
 
             # transfer and save name
             self.scp_client.put(path_zip.name, remote_path=str(sim_path_on_server))
             name_zip_file = Path(path_zip.name).name
 
-        # unzip (the reading of stout makes this call non-blocking!)
-        _stdin, _stdout, _stderr = self.ssh_client.exec_command(commands)
-        _mess = str(_stdout.read())
-        _err = str(_stderr.read())
-        if "Argument list too long" in _err:
-            print(
-                "Warning: Failed to delete old simulations on server."
-            )  # may happen, but not so tragic.
-        commands = (
-            "unzip -d " + str(sim_path_on_server) + " " + str(sim_path_on_server / name_zip_file)
+        # unzip
+        # here we should wait for finish
+        _stdin, stdout, _stderr = self.ssh_client.exec_command(
+            "unzip -o -d " + str(sim_path_on_server) + " " + str(sim_path_on_server / name_zip_file)
         )
-        _stdin, _stdout, _stderr = self.ssh_client.exec_command(commands)
-        _mess = str(_stdout.read())
-        _err = str(_stderr.read())
+        stdout.channel.set_combine_stderr(True)
+        output = stdout.readlines()
+
         # delete temp zip on the server
         self.ssh_client.exec_command("rm -f " + str(sim_path_on_server / name_zip_file))
 
@@ -336,7 +349,7 @@ class SimCon(object, metaclass=Singleton):
         """Collects the simulation data from the server.
 
         Parameters
-        -----------
+        ----------
         dut : DutView
         sweep : Sweep
         zip_result : bool, optional
@@ -393,7 +406,7 @@ class SimCon(object, metaclass=Singleton):
         """Collects the simulation log file from the server.
 
         Parameters
-        -----------
+        ----------
         dut : DutView
         sweep : Sweep
         """
@@ -417,7 +430,7 @@ class SimCon(object, metaclass=Singleton):
         """Runs all given simulations in parallel.
 
         Parameters
-        -----------
+        ----------
         sim_list :  [{}]
             List of dictionaries, each dictionary has a 'dut': :class:`~DMT.core.DutView` and 'sweep': :class:`~DMT.core.Sweep` key value pair.
 
@@ -471,12 +484,7 @@ class SimCon(object, metaclass=Singleton):
             if dut.simulate_on_server:
                 sims_to_zip.append({"dut": dut, "sweep": sweep})
 
-        print_progress_bar(
-            len(sim_list),
-            len(sim_list),
-            prefix="Preparing Simulations",
-            length=50,
-        )
+        print_progress_bar(len(sim_list), len(sim_list), prefix="Preparing Simulations", length=50)
         print("\n")  # new line after the progress bar
 
         if sims_to_zip:
@@ -669,7 +677,7 @@ class SimCon(object, metaclass=Singleton):
         """Starts the simulation
 
         Parameters
-        -----------
+        ----------
         dut : DutView
         sweep : Sweep
         """
@@ -679,7 +687,7 @@ class SimCon(object, metaclass=Singleton):
         )
         logging.debug("The simulation folder of this simulation is %s", sim_folder)
         log_file = open(sim_folder / "sim.log", "w")
-        log_file.write("The simulation command is\n" + dut.get_start_sim_command())
+        log_file.write(f"The simulation command is\n{dut.get_start_sim_command()}\n\n")
         return subprocess.Popen(
             dut.get_start_sim_command().split(),
             shell=False,
@@ -692,7 +700,7 @@ class SimCon(object, metaclass=Singleton):
         """Starts the remote simulation
 
         Parameters
-        -----------
+        ----------
         dut : DutView
         sweep : Sweep
         pbs : Boolean
@@ -752,7 +760,7 @@ def _check_simulation_needed(i_sim, n_tot, dut=None, sweep=None):
     sweep : DMT.core.Sweep
 
     Returns
-    --------
+    -------
     {key: DMT.core.Dataframe}
         In case the data is read from database or previous simulation.
     None
@@ -800,7 +808,7 @@ def _read_process_results(success, dut, sweep):
     sweep : DMT.core.Sweep
 
     Returns
-    --------
+    -------
     {'success': success, 'dut_hash':dut.get_hash(), 'data':dut.data}
     """
     dut_name = dut.name + str(dut.get_hash())
@@ -823,6 +831,7 @@ def _read_process_results(success, dut, sweep):
             print(
                 "{0:s}Simulation folder: {1:s} {2:s}".format(color_red, str(sim_folder), color_end)
             )
+            print((sim_folder / "sim.log").read_text())
             logging.info("Simulation of DuT %s with sweep %s failed.", dut_name, sim_name)
     except (SimulationUnsuccessful, FileNotFoundError, KeyError):
         color_red = "\033[91m"
@@ -833,6 +842,7 @@ def _read_process_results(success, dut, sweep):
             )
         )
         print("{0:s}Simulation folder: {1:s} {2:s}".format(color_red, str(sim_folder), color_end))
+        print((sim_folder / "sim.log").read_text())
         logging.info("Simulation of DuT %s with sweep %s failed.", dut_name, sim_name)
 
     return {"success": success, "dut_hash": dut.get_hash(), "data": dut.data}
