@@ -16,7 +16,9 @@ This can be used for Verilog files. The correct load of ADS is determined by fil
 
 """
 # DMT_core
-# Copyright (C) 2019  Markus Müller and Mario Krattenmacher and the DMT contributors <https://gitlab.hrz.tu-chemnitz.de/CEDIC_Bipolar/DMT/>
+# Copyright (C) from 2022  SemiMod
+# Copyright (C) until 2021  Markus Müller, Mario Krattenmacher and Pascal Kuthe
+# <https://gitlab.com/dmt-development/dmt-core>
 #
 # This file is part of DMT_core.
 #
@@ -32,6 +34,7 @@ This can be used for Verilog files. The correct load of ADS is determined by fil
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>
+from typing import Union
 import os
 import logging
 import copy
@@ -42,15 +45,14 @@ from DMT.config import COMMANDS
 from DMT.core import (
     DutCircuit,
     Circuit,
-    create_md5_hash,
     MCard,
     constants,
     DataFrame,
     specifiers,
     sub_specifiers,
-    Sweep,
-    McParameterComposition,
+    McParameterCollection,
 )
+from DMT.core.circuit import SGP_BJT, VOLTAGE, CURRENT, HICUML2_HBT, SHORT, DIODE
 
 from DMT.exceptions import SimulationUnsuccessful
 
@@ -59,7 +61,7 @@ class DutNgspice(DutCircuit):
     """Class description and methods
 
     Parameters
-    -----------
+    ----------
     database_dir    : string
         This is the directory were the DUT will create its database.
     name      :  string
@@ -116,50 +118,32 @@ class DutNgspice(DutCircuit):
             **kwargs,
         )
 
-    def get_hash(self):
-        """Returns a md5 hash generated from self.inp_header, if it is not set, this will return False!
-
-        Is overwritten here, to include the imported and appended files!
-
-        Returns
-        --------
-        str or False
-        """
-        if self.inp_header is None:
-            return False
-
-        if self._list_va_file_contents:
-            list_va = [content for (file_name, content) in self._list_va_file_contents]
-        else:
-            list_va = []
-        return create_md5_hash(self.inp_header, *self.list_copy, *list_va)
-
-    def create_inp_header(self, inp_circuit):
+    def create_inp_header(self, inp_circuit: Union[MCard, McParameterCollection, Circuit]):
         """Creates the input header of the given circuit description and returns it.
 
         Parameters
-        -----------
+        ----------
         input : MCard or Circuit
             If a HICUM modelcard is given, a common emitter Circuit is created from it.
 
         Returns
-        --------
+        -------
         netlist : str
         """
-        if isinstance(inp_circuit, MCard) or isinstance(inp_circuit, McParameterComposition):
+        if isinstance(inp_circuit, MCard) or isinstance(inp_circuit, McParameterCollection):
             # save the modelcard, in case it was set inderectly via the input header!
             self._modelcard = copy.deepcopy(inp_circuit)
             # generate inp_circuit for netlist generation
-            inp_circuit = inp_circuit.get_circuit()
+            inp_circuit = inp_circuit.get_circuit(**self.get_circuit_arguments)  # type: ignore
             # in case a standard circuit is used, this is the real input circuit
             self._inp_circuit = inp_circuit
-        elif not isinstance(inp_circuit, Circuit):
+        elif isinstance(inp_circuit, Circuit):
+            self._modelcard = None
+            self._inp_circuit = copy.deepcopy(inp_circuit)
+        else:
             raise OSError(
                 "For ADS circuits netlist generation is only possible from object of class DMT.classes.Circuit"
             )
-        else:
-            self._modelcard = None
-            self._inp_circuit = copy.deepcopy(inp_circuit)
 
         str_netlist = "DMT generated netlist\n"
         str_netlist += ".Options " + self._convert_dict_to_inp_line(self.simulator_options) + "\n"
@@ -170,7 +154,7 @@ class DutNgspice(DutCircuit):
             try:
                 if (
                     isinstance(inp_circuit, MCard)
-                    or isinstance(inp_circuit, McParameterComposition)
+                    or isinstance(inp_circuit, McParameterCollection)
                     and element.parameters.va_file is not None
                 ):
                     list_va_files.append(element.parameters.va_file)
@@ -180,11 +164,9 @@ class DutNgspice(DutCircuit):
         # load va files:
         for va_file in list_va_files:
             if self._copy_va_files:
-                with open(va_file) as my_va_file:
-                    va_file_content = my_va_file.read()
-                va_file = os.path.basename(va_file)
-                self._list_va_file_contents.append((va_file, va_file_content))
+                self._list_va_file_contents.append(va_file)
             else:
+                raise NotImplementedError("Absolute path of VA-File for NGSpice...")
                 if os.path.isfile(va_file):
                     # file is relative to current cwd -> transform to absolute path
                     va_file = os.path.abspath(va_file)
@@ -213,7 +195,7 @@ class DutNgspice(DutCircuit):
         sweep : Sweep
 
         Returns
-        --------
+        -------
         str
             header with added bias definitions
         """
@@ -264,53 +246,53 @@ class DutNgspice(DutCircuit):
         # output def
         # TODO: dirty fix to debug ngspice..
         # this is really messy...
-        device_out = [
-            "temp",
-            "m",
-            "vbe",
-            "vbc",
-            "vce",
-            "vsc",
-            "vbbp",
-            "ic",
-            "ib",
-            "ie",
-            "iavl",
-            "is",
-            "ibei",
-            "ibci",
-            "it",
-            "vbiei",
-            "vbpbi",
-            "vbici",
-            "vciei",
-            "rcx_t",
-            "re_t",
-            "rbi",
-            "rb",
-            "betadc",
-            "gmi",
-            "gms",
-            "rpii",
-            "rpix",
-            "rmui",
-            "rmux",
-            "roi",
-            "cpii",
-            "cpix",
-            "cmui",
-            "cmux",
-            "ccs",
-            "betaac",
-            "crbi",
-            "tf",
-            "ft",
-            "ick",
-            "p",
-            "tk",
-            "dtsh",
-        ]
-        str_netlist += ".save all " + " ".join(["@Q_Q_H[{:s}]".format(out) for out in device_out])
+        # device_out = [
+        #     "temp",
+        #     "m",
+        #     "vbe",
+        #     "vbc",
+        #     "vce",
+        #     "vsc",
+        #     "vbbp",
+        #     "ic",
+        #     "ib",
+        #     "ie",
+        #     "iavl",
+        #     "is",
+        #     "ibei",
+        #     "ibci",
+        #     "it",
+        #     "vbiei",
+        #     "vbpbi",
+        #     "vbici",
+        #     "vciei",
+        #     "rcx_t",
+        #     "re_t",
+        #     "rbi",
+        #     "rb",
+        #     "betadc",
+        #     "gmi",
+        #     "gms",
+        #     "rpii",
+        #     "rpix",
+        #     "rmui",
+        #     "rmux",
+        #     "roi",
+        #     "cpii",
+        #     "cpix",
+        #     "cmui",
+        #     "cmux",
+        #     "ccs",
+        #     "betaac",
+        #     "crbi",
+        #     "tf",
+        #     "ft",
+        #     "ick",
+        #     "p",
+        #     "tk",
+        #     "dtsh",
+        # ]
+        # str_netlist += ".save all " + " ".join(["@Q_Q_H[{:s}]".format(out) for out in device_out])
 
         # ngspice control statement
         str_netlist += "\n\n.control\n"
@@ -460,8 +442,8 @@ class DutNgspice(DutCircuit):
         Parameters
         ----------
         temp_def : float or dict
-            | If a single float is given, a constant temperature equal to it will be used.
-            | For a dict, a temperature sweep is added.
+            If a single float is given, a constant temperature equal to it will be used.
+            For a dict, a temperature sweep is added.
         """
         if isinstance(temp_def, (float, int)):
             # single temperature
@@ -704,7 +686,7 @@ class DutNgspice(DutCircuit):
             Sweep that has been simulated.
 
         Raises
-        --------
+        ------
         NotImplementedError
             If the Dut is not a simulatable dut.
         SimulationUnsuccessful
@@ -768,11 +750,11 @@ class DutNgspice(DutCircuit):
         into a line with "key1=value1 key2=value2 ". Correctly converts strings, iteratables, bools and numbers.
 
         Parameters
-        -----------
+        ----------
         dict_key_para : dict
 
         Returns
-        --------
+        -------
         str
             Line to add into input file
         """
@@ -799,7 +781,7 @@ class DutNgspice(DutCircuit):
         """Transforms a :class:`~DMT.classes.circuit.CircuitElement` into a string fitting for NGspice.
 
         Parameters
-        -----------
+        ----------
         circuit_element
             CircuitElement to transform
 
@@ -808,22 +790,25 @@ class DutNgspice(DutCircuit):
         str
             Netlist line
         """
-        if len(circuit_element.name) != 3:
-            raise OSError(
-                "To enable data_reader.read_ADS_bin, all element names have to be 3 characters long. Given was: "
-                + circuit_element.name
-            )
+        # if len(circuit_element.name) != 3:
+        #     raise OSError(
+        #         "To enable data_reader.read_ADS_bin, all element names have to be 3 characters long. Given was: "
+        #         + circuit_element.name
+        #     )
 
         # map dmt circuit element_type to ngspice element types
         # only those that differ between DMT definiton and NGspice definition are listed here
         element_types = {
-            "V_Source": "V",
-            "I_Source": "I",
+            VOLTAGE: "V",
+            CURRENT: "I",
             "hicumL2va": "Q",
+            HICUML2_HBT: "Q",
+            SGP_BJT: "Q",
+            "bjtn": "Q",
         }
         if circuit_element.element_type in element_types.keys():
             element_type = element_types[circuit_element.element_type]
-        elif circuit_element.element_type == "Short":
+        elif circuit_element.element_type == SHORT:
             element_type = "V"
             circuit_element.name = "V_" + circuit_element.name.replace("I", "V")
             circuit_element.parameters = [("dc", str(0)), ("ac", str(0))]
@@ -834,26 +819,84 @@ class DutNgspice(DutCircuit):
         else:
             element_type = circuit_element.element_type
 
-        additional_str = ""
-        if element_type == "Q":
-            additional_str = "hicum_va"
-
-        str_netlist = (
-            element_type
-            + "_"
-            + circuit_element.name
-            + " "
-            + " ".join(circuit_element.contact_nodes)
-            + " "
-            + additional_str
-            + " "
+        str_netlist = f"{element_type}_{circuit_element.name} " + " ".join(
+            circuit_element.contact_nodes
         )
         if circuit_element.parameters is not None:
-            try:
+            if isinstance(circuit_element.parameters, MCard):
+                str_temp = "+ "
+
+                if circuit_element.element_type in ["hicumL2va", HICUML2_HBT]:
+                    mcard = circuit_element.parameters
+                    str_instance_parameters = ""
+                    str_model_parameters = ""
+                    str_type = "NPN"
+                    for para in sorted(mcard.paras, key=lambda x: (x.group, x.name)):
+                        if para.name == "type":
+                            str_type = "NPN" if (para.value == 1) else "PNP"
+                        elif para.name in [
+                            "dt",
+                        ]:  # here all instance parameters
+                            str_instance_parameters += "{0:s}={0:10.10e} ".format(para)
+                        else:  # here all model parameters
+                            str_model_parameters += "{0:s}={0:10.10e} ".format(para)
+
+                    for (key, val) in self.initial_conditions.items():
+                        # dirty to allow debugging ngspice
+                        str_model_parameters += "{0:s}={1:10.10e} ".format(key, val)
+                    str_temp = (
+                        f"hicum_va {str_instance_parameters}\n"  # we should count here somehow the models
+                        + f".model hicum_va {str_type} level=8\n"
+                        + f"+ {str_model_parameters}"
+                    )
+                elif circuit_element.element_type in [SGP_BJT, "bjtn"]:
+                    mcard = circuit_element.parameters
+                    str_instance_parameters = ""
+                    str_model_parameters = ""
+                    str_type = "NPN"
+                    for para in sorted(mcard.paras, key=lambda x: (x.group, x.name)):
+                        if para.name == "type":
+                            str_type = "NPN" if (para.value == 1) else "PNP"
+                        elif para.name in []:  # here all instance parameters
+                            str_instance_parameters += "{0:s}={0:10.10e} ".format(para)
+                        else:  # here all model parameters
+                            str_model_parameters += "{0:s}={0:10.10e} ".format(para)
+
+                    for (key, val) in self.initial_conditions.items():
+                        # dirty to allow debugging ngspice
+                        str_model_parameters += "{0:s}={1:10.10e} ".format(key, val)
+                    str_temp = (
+                        f"QMOD {str_instance_parameters}\n"  # we should count here somehow the models
+                        + f".model QMOD {str_type} level=1\n"
+                        + f"+ {str_model_parameters}"
+                    )
+                elif circuit_element.element_type in [DIODE]:
+                    mcard = circuit_element.parameters
+                    str_instance_parameters = ""
+                    str_model_parameters = ""
+                    str_type = "NPN"
+                    additional_str = ""
+                    for para in sorted(mcard.paras, key=lambda x: (x.group, x.name)):
+                        if para.name == "osdi":
+                            if para - value == 1:
+                                additional_str = " osdi " + circuit_element.name
+                        str_model_parameters += "{0:s}={0:10.10e} ".format(para)
+
+                    str_temp = f"\n.model dmod {additional_str} d( {str_model_parameters} )"  # we should count here somehow the models
+                else:
+                    raise NotImplementedError(
+                        f"The element type {circuit_element.element_type} is not implemented for ngspice.",
+                        "Check the ngspice manual if this type needs special treatment and implement it accordingly.",
+                    )
+            else:
                 str_temp = []
                 for (para, value) in circuit_element.parameters:
                     if para in ["C", "R", "L"]:  # rename according to ngspice manual
                         str_temp.append(value)
+                    elif para in ["Vdc", "Vac"] and not isinstance(
+                        value, float
+                    ):  # just leave voltages from lines, as ngpsice directly changes the sources and not the parameters
+                        pass
                     else:
                         str_temp.append(para + "=" + value)
 
@@ -871,57 +914,18 @@ class DutNgspice(DutCircuit):
                     sim_paras = "\n.param " + sim_paras
 
                 str_temp = str_temp + "".join(sim_paras)
-            except TypeError:
-                str_temp = "+ "
-                # for para in circuit_element.parameters.iter_alphabetical():
-                #     if para.name[0] != '_': #do not use parameters that start with _
-                #         str_temp = str_temp + "  {0:s} = {0:10.10e}".format( para)
-
-                if True:
-                    mcard = circuit_element.parameters
-                    str_instance_parameters = ""
-                    str_model_parameters = ""
-                    for para in sorted(mcard.paras, key=lambda x: (x.group, x.name)):
-                        if para.name == "type":
-                            str_type = "NPN" if (para.value == 1) else "PNP"
-                        elif para.name in [
-                            "dt",
-                        ]:  # here all instance parameters
-                            str_instance_parameters += "{0:s}={0:10.10e} ".format(para)
-                        else:  # here all model parameters
-                            str_model_parameters += "{0:s}={0:10.10e} ".format(para)
-
-                    for (
-                        key,
-                        val,
-                    ) in self.initial_conditions.items():  # dirty to allow debugging ngspice
-                        str_model_parameters += "{0:s}={1:10.10e} ".format(key, val)
-                    str_temp = (
-                        str_instance_parameters
-                        + "\n.model hicum_va "
-                        + str_type
-                        + " level=8\n+ "
-                        + str_model_parameters
-                    )
-                else:
-                    str_temp += circuit_element.parameters.print_parameters()
-
-                    if "hicumL2V2p4p0" in circuit_element.parameters.va_file:
-                        str_temp = "\n.model hicum_va npn level=10\n" + str_temp
-                    else:  # hicum InP
-                        str_temp = "\n.model hicum_va npn level=11\n" + str_temp
 
+            # catch ngspice keywords
+            str_temp = str_temp.replace(" L=", "")
+            str_temp = str_temp.replace("Vdc=", "dc ")
+            str_temp = str_temp.replace("Idc=", "dc ")
+            str_temp = str_temp.replace("Vac=", "ac ")
+            str_temp = str_temp.replace("Iac=", "ac ")
         else:
             str_temp = ""
 
-        # catch ngspice keywords TODO: move this directly to parameter writing
-        str_temp = str_temp.replace(" L=", "")
-        str_temp = str_temp.replace("Vdc=", "dc ")
-        str_temp = str_temp.replace("Idc=", "dc ")
-        str_temp = str_temp.replace("Vac=", "ac ")
-        str_temp = str_temp.replace("Iac=", "ac ")
         str_netlist = str_netlist.replace("IS", "I_")
-        return str_netlist + str_temp + "\n"
+        return str_netlist + " " + str_temp + "\n"
 
     def join(self, dfs):
         """Join DC and AC dataframes into one dataframe"""
