@@ -1,7 +1,9 @@
 """ The interface to the Xyce circuit simulator from Sandia
 """
 # DMT_core
-# Copyright (C) 2019  Markus Müller and Mario Krattenmacher and the DMT contributors <https://gitlab.hrz.tu-chemnitz.de/CEDIC_Bipolar/DMT/>
+# Copyright (C) from 2022  SemiMod
+# Copyright (C) until 2021  Markus Müller, Mario Krattenmacher and Pascal Kuthe
+# <https://gitlab.com/dmt-development/dmt-core>
 #
 # This file is part of DMT_core.
 #
@@ -19,18 +21,19 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>
 import copy
 import logging
-from os import path
 import re
 import subprocess
 import warnings
 import numpy as np
 import pandas as pd
+import time
 from pathlib import Path
 
 from DMT.config import DATA_CONFIG
 from DMT.core import (
     DutCircuit,
     DutType,
+    McParameterCollection,
     MCard,
     Sweep,
     SweepDef,
@@ -68,6 +71,7 @@ class DutXyce(DutCircuit):
         simulator_command=None,
         simulator_arguments=None,
         simulator_options=None,
+        build_xyce_plugin_command="buildxyceplugin",
         **kwargs,
     ):
         if simulator_command is None:
@@ -81,6 +85,7 @@ class DutXyce(DutCircuit):
         if simulator_arguments is None:
             simulator_arguments = []
 
+        self.build_xyce_plugin_command = build_xyce_plugin_command
         self._va_plugins_to_compile = []
 
         super().__init__(
@@ -106,15 +111,16 @@ class DutXyce(DutCircuit):
         """Creates the input header of the given circuit description and returns it.
 
         Parameters
-        -----------
+        ----------
         input : DMT.core.Circuit or DMT.core.MCard
             If a MCard is given, the circuit is created from it.
 
         Returns
-        --------
+        -------
         netlist : str
         """
-        if isinstance(inp_circuit, MCard):
+        va_from_model_build_in = False
+        if isinstance(inp_circuit, MCard) or isinstance(inp_circuit, McParameterCollection):
             # save the modelcard, in case it was set inderectly via the input header!
             self._modelcard = copy.deepcopy(inp_circuit)
             # get the circuit for netlist generation
@@ -124,16 +130,21 @@ class DutXyce(DutCircuit):
             ####################################################
 
             # in case a standard circuit is used, this is the real input circuit
-            self._inp_circuit = inp_circuit.get_circuit()
-        elif not isinstance(inp_circuit, Circuit):
+            self._inp_circuit = inp_circuit.get_circuit(**self.get_circuit_arguments)  # type: ignore
+            if (
+                "use_build_in" in self.get_circuit_arguments
+                and self.get_circuit_arguments["use_build_in"]
+            ):
+                va_from_model_build_in = True
+        elif isinstance(inp_circuit, Circuit):
+            self._modelcard = None
+            self._inp_circuit = copy.deepcopy(inp_circuit)
+        else:
             raise OSError(
                 "For Xyce circuits netlist generation is only possible from object of class DMT.classes.Circuit. Passed "
                 + str(inp_circuit)
                 + "."
             )
-        else:
-            self._modelcard = None
-            self._inp_circuit = copy.deepcopy(inp_circuit)
 
         str_netlist = (
             "DMT Xyce simulation\n"
@@ -143,57 +154,44 @@ class DutXyce(DutCircuit):
         )
 
         # is a modelcard inside the netlist?
-        list_va_files = set()  # so we have each element only once
+        list_va_files = list()
         for element in self._inp_circuit.netlist:
             try:
-                list_va_files.add(element.parameters.va_file)
+                list_va_files.append(element.parameters.va_codes)
             except AttributeError:
                 # element does not have a va_file.
                 pass
 
-        # load va file plugins and mark missing ones -> they are compiled later:
-        va_plugins = []
-        self._va_plugins_to_compile = []  # reset the plugins
-        for va_file in list_va_files:
-            if va_file is None:
-                continue
+        if not va_from_model_build_in:
+            # load va file plugins and mark missing ones -> they are compiled later:
+            va_plugins = []
+            self._va_plugins_to_compile = []  # reset the plugins
+            for vafile in list_va_files:
+                if vafile is None:
+                    continue
 
-            if not isinstance(va_file, Path):
-                va_file = Path(va_file)
-
-            if self._copy_va_files:
-                va_file_content = va_file.read_text()
-                self._list_va_file_contents.append((va_file.name, va_file_content))
-
-            if va_file.is_file():
-                # file is relative to current cwd -> transform to absolute path
-                va_file = va_file.resolve()
-                va_file_content = va_file.read_text()
+                if self._copy_va_files:
+                    self._list_va_file_contents.append(vafile)
 
                 # hash file content
-                va_hash = create_md5_hash(va_file_content)
+                va_hash = vafile.get_tree_hash()
                 # check if plugin is already compiled and in the folder
                 path_va_hash = self.sim_dir / "xyce_plugins" / (va_hash + ".so")
 
                 if not path_va_hash.is_file():
                     # if not: add to "to_compile"
-                    self._va_plugins_to_compile.append((path_va_hash, va_file_content))
+                    self._va_plugins_to_compile.append((path_va_hash, vafile))
 
                 va_plugins.append(path_va_hash)
-            else:
-                raise IOError(
-                    "DutXyce: Was not able to locate the Verilog-AMS file. The given path was \n%s",
-                    va_file,
-                )
 
-        # add to simulator arguments
-        if va_plugins:
-            try:
-                i_plugin = self.sim_args.index("-plugin")
-                self.sim_args[i_plugin + 1] = ",".join(str(va_path) for va_path in va_plugins)
-            except ValueError:
-                self.sim_args.append("-plugin")
-                self.sim_args.append(",".join(str(va_path) for va_path in va_plugins))
+            # add to simulator arguments
+            if va_plugins:
+                try:
+                    i_plugin = self.sim_args.index("-plugin")
+                    self.sim_args[i_plugin + 1] = ",".join(str(va_path) for va_path in va_plugins)
+                except ValueError:
+                    self.sim_args.append("-plugin")
+                    self.sim_args.append(",".join(str(va_path) for va_path in va_plugins))
 
         str_netlist += "\n* Netlist\n"
 
@@ -218,7 +216,7 @@ class DutXyce(DutCircuit):
         """Transforms a :class:`~DMT.classes.circuit.CircuitElement` into a string fitting for ADS.
 
         Parameters
-        -----------
+        ----------
         circuit_element : :class:`DMT.core.CircuitElement`
             CircuitElement to transform
         index : int
@@ -254,8 +252,14 @@ class DutXyce(DutCircuit):
                 parameters = "DC {" + dc + "}"
             try:
                 ac = next(para for para in circuit_element.parameters if para[0] == "Vac")[1]
+                try:
+                    float_ac = float(ac)
+                    ac = ac + "V"
+                except ValueError:
+                    # if dc contaíns a string -> it is a parameter and hence no unit needed but braces...
+                    ac = "{" + ac + "}"
                 phase = str(np.rad2deg(0))  # just to remember this
-                parameters += " AC " + ac + "V " + phase
+                parameters += " AC " + ac + " " + phase
             except StopIteration:
                 pass
             return (
@@ -392,7 +396,7 @@ class DutXyce(DutCircuit):
         sweep : Sweep
 
         Returns
-        --------
+        -------
         str
             header with added bias definitions
         """
@@ -403,23 +407,28 @@ class DutXyce(DutCircuit):
         # compile plugins if needed and also add to arguments
         if self._va_plugins_to_compile:
             print_progress_bar(
-                0,
-                len(self._va_plugins_to_compile),
-                prefix="Compiling Xyce plugins",
-                length=50,
+                0, len(self._va_plugins_to_compile), prefix="Compiling Xyce plugins", length=50
             )
             processes = []
-            for path_plugin, va_content in self._va_plugins_to_compile:
+            for path_plugin, vafile in self._va_plugins_to_compile:
                 # write to location
-                path_plugin.parent.mkdir(exist_ok=True)
-                path_va = path_plugin.with_suffix(".va")
-                path_va.write_text(va_content)
+                vafile.rename_root(path_plugin.with_suffix(".va").name)
+                # does it exist now ? (can happen if two different duts are used...)
+                if path_plugin.is_file():
+                    continue
+                vafile.write_files(path_plugin.parent)
 
-                # call "buildxyceplugin"
+                # call "buildxyceplugin.sh"
                 log_file = open(path_plugin.parent / (path_plugin.stem + "_compile.log"), "w")
                 processes.append(
                     subprocess.Popen(
-                        ["buildxyceplugin", "-o", path_plugin.stem, path_va.name, "."],
+                        [
+                            self.build_xyce_plugin_command,
+                            "-o",
+                            path_plugin.stem,
+                            vafile.root,
+                            ".",
+                        ],
                         shell=False,
                         cwd=path_plugin.parent,
                         stderr=subprocess.STDOUT,
@@ -427,16 +436,21 @@ class DutXyce(DutCircuit):
                     )
                 )
 
-            while True:
+            time_start = time.time()
+            time_out = True
+            while time.time() - time_start < 60:
                 finished = 0
                 for i_p, process in enumerate(processes[:]):
                     returncode = process.poll()
-                    if returncode is not None:
+                    if returncode is None:
+                        time.sleep(0.1)
+                    else:
                         if (
                             returncode == 0
-                            and returncode != 134
-                            and returncode != 139
-                            and returncode != 1
+                            or returncode != 0
+                            # and returncode != 134
+                            # and returncode != 139
+                            # and returncode != 1
                         ):
                             print_progress_bar(
                                 finished,
@@ -444,12 +458,25 @@ class DutXyce(DutCircuit):
                                 prefix="Compiling Xyce plugins",
                                 length=50,
                             )
+                            logpath = Path(process.args[3][:-3] + "_compile.log")
                             del processes[i_p]
                             finished += 1
+                        else:
+                            logpath = Path(process.args[3][:-3] + "_compile.log")
+                            raise OSError(
+                                "The plugin was not compiled successfull! Check the log in ",
+                                str(logpath),
+                                "content:",
+                                logpath.with_name("buildxyceplugin.log").read_text(),
+                            )
 
                 if not processes:
+                    time_out = False
                     break
 
+            if time_out:
+                raise OSError("The plugin was not compiled in time!")
+
             print("\n")  # new line after the progress bar
 
         self._va_plugins_to_compile = []
@@ -531,7 +558,8 @@ class DutXyce(DutCircuit):
             str_netlist += ".AC DATA=TAB_FREQUENCIES \n"
             str_netlist += ".DATA TAB_FREQUENCIES \n"
             str_netlist += "+ FREQ \n"
-            str_netlist += "+ " + " ".join("{:.6g}".format(val) for val in swd_ac.values) + " \n"
+            str_netlist += "+ " + " ".join(f"{val:.6g}" for val in swd_ac.values) + " \n"
+            str_netlist += ".ENDDATA \n"
             str_netlist += "* AC output definition\n"
             str_netlist += ".PRINT AC FORMAT=CSV FILE=AC.csv V(*) I(*) \n"  # TODO add outputdef ?!?
         else:
@@ -554,7 +582,7 @@ class DutXyce(DutCircuit):
             Sweep that has been simulated.
 
         Raises
-        --------
+        ------
         NotImplementedError
             If the Dut is not a simulatable dut.
         SimulationUnsuccessful
@@ -591,7 +619,7 @@ class DutXyce(DutCircuit):
             If True, the simulation folder is deleted after reading.
 
         Raises
-        -------
+        ------
         NotImplementedError
             If the Dut is not a simulatable dut.
         IOError
@@ -608,6 +636,7 @@ class DutXyce(DutCircuit):
             filepaths = [
                 Path(sweep),
             ]
+            sim_folder = filepaths[0].parent
             if key is None:
                 warnings.warn("DMT->DutXyce->import_output_data: Added a df with the key None!")
 
@@ -785,13 +814,7 @@ class DutXyce(DutCircuit):
 
             # get all AC columns, except frequency and ac_switch
             cols_ac_old = [
-                col
-                for col in df_ac.columns
-                if col
-                not in [
-                    specifiers.FREQUENCY,
-                    col_ac_switch,
-                ]
+                col for col in df_ac.columns if col not in [specifiers.FREQUENCY, col_ac_switch]
             ]
 
             #  the AC columns to account for forward and backward simulation
@@ -799,6 +822,8 @@ class DutXyce(DutCircuit):
                 ports = ["B", "C"]
             elif self.dut_type.is_subtype(DutType.flag_mos):
                 ports = ["G", "D"]
+            else:
+                raise NotImplementedError("DutType not known!")
             cols_ac_forward = []
             cols_ac_backward = []
             for col_old in cols_ac_old:
@@ -813,14 +838,7 @@ class DutXyce(DutCircuit):
                     + col_old.sub_specifiers
                 )
             # get the DC columns
-            cols_dc = [
-                col
-                for col in df_dc.columns
-                if col
-                not in [
-                    col_ac_switch,
-                ]
-            ]
+            cols_dc = [col for col in df_dc.columns if col not in [col_ac_switch]]
 
             # find the row pairs with same forced dc, temp and frequency
             cols_filter = [
@@ -868,13 +886,16 @@ class DutXyce(DutCircuit):
                 series_ac_forward[specifiers.FREQUENCY] = df_main.loc[i_row_1, specifiers.FREQUENCY]
 
                 # add the new row to the new dataframe
-                df_main_new = df_main_new.append(
-                    series_dc.append([series_ac_forward, series_ac_backward]), ignore_index=True
+                df_main_new = pd.concat(
+                    [
+                        df_main_new,
+                        pd.concat([series_dc, series_ac_forward, series_ac_backward]).to_frame().T,
+                    ],
+                    axis="index",
+                    ignore_index=True,
                 )
 
-            df_main = df_main_new.sort_values(
-                by=cols_filter,
-            )
+            df_main = df_main_new.sort_values(by=cols_filter)
 
         # unit converts!
         if specifiers.TEMPERATURE in df_main.columns:
